## Auflisten der Aufgaben und beabsichtigter Lösungsschritte

### Allgemeine Zielsetzung und Aufteilung

In dem Projekt soll die Frage geklärt werden, ob Chatbots sich "selbst trainieren können". Dazu soll sich mit der Open-Source Software [Rasa](https://rasa.com) auseinandergesetzt werden.

Der Chatbot im Beispiel soll englischsprachig arbeiten und Fragen zu berühmten Physikern beantworten.

Die Aufgaben werden grob in drei Bereiche aufgeteilt: Die Akquise und Aufbereitung von Daten, die Konfiguration des Rasa-Chatbots und die Dokumentation des Projekts

### Akquise und Aufbereitung von Daten zu Physikern

#### Zielsetzung

Für den Chatbot sollen Daten zu Physikern auf Englisch gewonnen werden. Eine mögliche Quelle ist Wikipedia. Von dort wurde bereits eine Liste mit etwa [1000 Physikernamen](https://en.wikipedia.org/wiki/List_of_physicists) gewonnen. Zu den Namen sollen die Wikipedia-Artikel als HTML akquiriert werden.
Die so gewonnenen Daten sollen dann in R für die Intents, mit denen der Chatbot arbeiten soll (siehe unten), aufbereitet werden.
Im Idealfall werden R-Scripts geschrieben, die fertige Rasa-Trainingsfiles ausgeben.

#### Aufgaben
* Englischsprachige Wikipedia-Einträge zu berühmten Physikern gewinnen
* Die Physikerdaten in Tabellenform speichern
* Die Daten cleanen (Entfernung von Tags, Hyperlinks etc)
* Für die Intents Skripte erstellen, um die benötigten Informationen zu extrahieren.
    * Dazu gegebenenfalls weitere Aufbereitung (Named Entity Recognition, Part of Speech Tagging, Tokenization)
* Gewonnene Informationen in ein zu Rasa kompatibles Datenformat bringen

#### Arbeitsstand
* Es sind bereits 983 Physikernamen runtergeladen:
  ```
  Jules Aarons
  Ernst Karl Abbe
  Derek Abbott
  Hasan Abdullayev
  Alexei Alexeyevich Abrikosov
  Robert Adler
  Stephen L. Adler
  Franz Aepinus
  ```
* Die Wikipedia-Artikel zu allen Namen sind in einem XML-File gespeichert.
  > Beispiel einfügen

* Es wurde ein R-Skript geschrieben, das Namen und zugehörige Wikipedia-Artikel in einem `.csv` File speichert
  ```
  library(xml2)
   s
  data <- read_xml("../data/Wikipedia-20181120103842.xml")
  
  title.nodes <- xml_find_all(data, ".//title")
  
  titles <- sapply(title.nodes, xml_text)
  
  text.nodes <- xml_find_all(data, ".//text")
  
  texts <- sapply(text.nodes, xml_text)
  
  df.out <- data.frame(Title = titles,
                       Text = texts)
  
  saveRDS(df.out, "../data/texte.RDS")
  
  write.table(df.out, "../data/texte.csv")
  
  ```

### Konfiguration des Rasa-Chatbots

Anmerkung: Dieser Aufgabenbereich wird in der ersten Phase des Projekts (Dezember 2018) hinter die Datenakquise zurückgestellt. Der Bot soll zunächst so einfach wie möglich implementiert werden.


#### Zielsetzung

Es soll ein [Rasa Chatbot](https://rasa.com/docs/) entwickelt werden, der auf Englisch Fragen zu Physikern beantwortet. Dazu sollen die Rasa-Technologien Rasa NLU (Natural Language Understanding) und Rasa Core verwendet werden. 
Rasa Core soll dafür genutzt werden, dass Rasa die Fragen und dahinterliegenden Intents (Absichten) eines Chatbot-Nutzers versteht. Rasa Core ist für die Antwortgenerierung zuständig und steuert den Gesprächsverlauf. 
Um beide Teile erfolgreich zu implementieren, soll sich mit der Dokumenation von Rasa auseinandergesetzt werden. Besonderes Augenmerk liegt auf der effizienten Einspeisung der aufbereitenten Physikerdaten in den Rasa-Bot.
Im Laufe des Projekts soll zunächst ein Chatbot erstellt werden, der durch die Teammitglieder trainiert wird und einfach Fragen beantworten kann. (5 Intents siehe unten). Im weiteren Verlauf des Projekts soll geklärt werden, inwieweit der Bot sich selbst trainieren kann. Dazu sollen zB Rasa Custom Actions über rasa_core_sdk oder die [Erstellung von Bot Antworten](https://rasa.com/docs/core/responses/) über einen [externen Server](https://rasa.com/docs/core/customactions/) in Betracht gezogen werden.

#### Aufgaben
* Lauffähige Version des Rasa-Bots erstellen
* Intents für den Bot ausformulieren
* Den Bot mit den erstellten Intents trainieren
* Slots in den Intents nutzen, damit der Bot 
* Den Bot mit aufbereiteten Physikerdaten trainieren

#### Arbeitsstand

* Der Rasa Bot läuft auf den Notebooks aller Teilnehmer in `pip` oder `miniconda`-Umgebungen
* Für den Physiker-Prototypen sollen folgenden Intents implementiert werden:
    * _birth_ - "Where and when was $name born?"
    * _isAlive_ - "Is $name still alive?"
    * _education_ - "Where did $name go to school?"
    * _researchArea_ - "What did $name discover?"
    * _hasNobelPrize_ - "Did $name win the Nobel Prize?"


### Dokumentation des Projekts